#ifndef _IDSA_RULE_H_
#define _IDSA_RULE_H_

#define RULE_GATE_NONE       0x0000
#define RULE_GATE_NEGATE     0x0001
#define RULE_GATE_AND        0x0002
#define RULE_GATE_OR         0x0004

#define RULE_GATE_NAND       RULE_GATE_AND|RULE_GATE_NEGATE
#define RULE_GATE_NOR        RULE_GATE_OR|RULE_GATE_NEGATE

#define RULE_FIRE_CLEAR      0x0000
#define RULE_FIRE_TRUE       0x0001
#define RULE_FIRE_FALSE      0x0002

#include "types.h"

struct rule_match{
  int m_gate;     /* what operation to perform */
  int m_fire;     /* has the rule been triggered ? */
  struct rule_match *m_up,*m_left,*m_right;
};

#include <idsa.h>

#define RULE_ACTION_NULL     0x0000
#define RULE_ACTION_ALLOW    0x0001  /* allow */
#define RULE_ACTION_DENY     0x0002  /* don't allow */
#define RULE_ACTION_CONTINUE 0x0004  /* continue evaluation of rule list */
#define RULE_ACTION_DROP     0x0008  /* close connection */
#define RULE_ACTION_SEND     0x0010  /* send unit to other side */
#define RULE_ACTION_LOG      0x0020  /* send a log message somewhere */
#define RULE_ACTION_KEEP     0x0040  /* add a unit to one of the variables */

struct rule_action{
  int a_type;

  /* these three lines could be a union, but are not */
  union {
    LOG_OUTPUT *a_output;
    IDSA_UNIT *a_unit;
    QUEUE_KEEP *a_keep;
  } a_un;

  struct rule_action *a_next;
};

struct rule_body{
  int r_number;
  int r_policy;
  struct rule_match *r_match;
  struct rule_action *r_action;
  struct rule_body *r_next;
};

void rule_clear(RULE_BODY *s);
int rule_run(STATE_SET *s, RULE_BODY *r, IDSA_EVENT *em, IDSA_EVENT *er, JOB *j);
RULE_BODY *rule_new();
void rule_free(RULE_BODY *r);

RULE_MATCH *match_new(int g);
int match_fire_true(RULE_MATCH *m);
int match_fire_false(RULE_MATCH *m);
void match_free(RULE_MATCH *m);

RULE_ACTION *action_new(int t);
void action_free(RULE_ACTION *a);

#include <stdio.h>

void rule_dump(RULE_BODY *r, FILE *f);

#endif
