#ifndef _IDSA_SET_H_
#define _IDSA_SET_H_

#include <sys/utsname.h>
#include <sys/types.h>

#include <idsa_internal.h>

#include "rule.h"
#include "table.h"
#include "output.h"
#include "job.h"
#include "queue.h"

struct state_set{
  int s_policy;            /* return code of trigger */
  UNION_BRANCH *s_branch;  /* list of entities triggered */

  RULE_BODY *s_rules;      /* all the rules */
  RULE_BODY *s_currentrule;/* transient last rule */
  int s_rulecount;         /* counter to number rules */

  QUEUE_VAR *s_queue;      /* variable queues */

  LOG_OUTPUT *s_output;    /* all the sinks */
  int s_outputfail;        /* problems with output */

  JOB *s_jobs;             /* table of client connections */
  int s_jobmax;            /* size of table */
  int s_jobcount;          /* number of entries used */

  char *s_hostname;        /* cached hostname */
  gid_t s_gid;             /* cached gid */

  IDSA_EVENT *s_template;  /* template for internal events */
  IDSA_EVENT *s_internal;  /* internal event */

  IDSA_EVENT *s_message;   /* event received from client */
  IDSA_EVENT *s_reply;     /* event sent to client */

  int s_errno;
  char s_errbuf[IDSA_M_LONG];
};

STATE_SET *set_new();
void set_free(STATE_SET *s);

void set_clear(STATE_SET *s);
int set_do(STATE_SET *s, IDSA_EVENT *em, IDSA_EVENT *er, JOB *j);

int set_drop(STATE_SET *s);

#endif
